#ifndef HELPERS_H
#define HELPERS_H

#include <cmath>
#include <vector>
#include <algorithm>

using namespace std;

inline double CosUhla(double x1, double y1, double x2, double y2)
{
	return (x1*x2+y1*y2)/(sqrt(x1*x1+y1*y1)*sqrt(x2*x2+y2*y2));
}

inline double SinUhla(double x1, double y1, double x2, double y2)
{
	return (x1*y2-y1*x2)/(sqrt(x1*x1+y1*y1)*sqrt(x2*x2+y2*y2));
}

inline void GetHitedFields(int y, int x, int sy, int sx, vector<pair<int,int> >&hits)
{
	int s, t;
	if(sx==0)
	{
		if(sy < 0) s = -1;
		else s = 1;
		for(int i = y; i != y+sy; i+=s)
		{
			hits.push_back(pair<int,int>(i, x));
		}
		hits.push_back(pair<int,int>(y+sy, x));
	}
	else if(sy==0)
	{
		if(sx < 0) s = -1;
		else s = 1;
		for(int i = x; i != x+sx; i+=s)
		{
			hits.push_back(pair<int,int>(y, i));
		}
		hits.push_back(pair<int,int>(y, x+sx));
	}
	else if(abs(sy)==abs(sx))
	{
		if(sy < 0) s=-1;
		else s = 1;
		if(sx < 0) t=-1;
		else t = 1;
		int i=y, j=x;
		while(i!=y+sy)
		{
			hits.push_back(pair<int,int>(i, j));
			hits.push_back(pair<int,int>(i+s, j));
			hits.push_back(pair<int,int>(i, j+t));
			i+=s;
			j+=t;
		}
		hits.push_back(pair<int,int>(y+sy,x+sx));
	}
	else if(abs(sx) > abs(sy) && abs(sx)==2*abs(sy))
	{
		if(sy < 0) s=-1;
		else s = 1;
		if(sx < 0) t=-1;
		else t = 1;
		int i=y, j=x;
		while(i!=y+sy)
		{
			hits.push_back(pair<int,int>(i, j));
			hits.push_back(pair<int,int>(i, j+t));
			hits.push_back(pair<int,int>(i+s, j+t));
			i+=s;
			j+=t+t;
		}
		hits.push_back(pair<int,int>(y+sy,x+sx));
	}
	else if(abs(sx) < abs(sy) && abs(sy)==2*abs(sx))
	{
		if(sy < 0) s=-1;
		else s = 1;
		if(sx < 0) t=-1;
		else t = 1;
		int i=y, j=x;
		while(i!=y+sy)
		{
			hits.push_back(pair<int,int>(i, j));
			hits.push_back(pair<int,int>(i+s, j));
			hits.push_back(pair<int,int>(i+s, j+t));
			i+=s+s;
			j+=t;
		}
		hits.push_back(pair<int,int>(y+sy,x+sx));
	}
}

#endif