#include <string>
#include <iostream>

#ifdef WIN32
	#include <winsock2.h>
	#include <ws2tcpip.h>
	#include <windows.h>
#else
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <sys/socket.h>
	#include <sys/poll.h>
	#include <errno.h>
#endif

#include "network.h"


using namespace std;

bool first_time=true;
#ifdef WIN32
void first_time_foo(){
	if(first_time){
		first_time=false;
		WSADATA wsaData;
		WSAStartup(MAKEWORD( 2, 2 ),&wsaData);
	}
}
#endif

// Create connection from existing descriptor
Connection::Connection(int fd)
{
#ifdef WIN32
	first_time_foo();
#endif
	this->fd = fd;
	async_recv_len = -1;
}

// Empty connection, which can connect with 'connect' method
Connection::Connection()
{
#ifdef WIN32
	first_time_foo();
#endif
	this->fd = -1;
	async_recv_len = -1;
}

Connection::~Connection()
{
	if (fd != -1) {
#ifdef WIN32
		shutdown(fd,SD_BOTH);
		closesocket(fd);
#else
		shutdown(fd,SHUT_RDWR);
		close(fd);
#endif
		fd=-1;
	};
}

int Connection::connect(string host, int port)
{
#ifdef WIN32
	fd= socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
	if(fd == INVALID_SOCKET){
		perror("socket");
		return -1;
	}
	struct sockaddr_in addr;
	addr.sin_family=AF_INET;
	addr.sin_addr.s_addr=inet_addr(host.c_str());
	addr.sin_port=htons(port);
	int	res = ::connect(fd,(SOCKADDR*)&addr,sizeof(addr));
	if(res == SOCKET_ERROR){
		perror("connect");
		return -1;
	}

#else
	struct sockaddr_in addr;
	if (inet_pton(AF_INET, host.c_str(), &addr.sin_addr.s_addr) == -1) {
		perror("inet_pton");
		return -1;
	};

	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;

	fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd == -1) {
		perror("socket");
		return -1;
	};

	if (::connect(fd, (struct sockaddr*)&addr, sizeof(addr))) {
		perror("connect");
		return -1;
	};
#endif
	return 0;
}

#ifdef WIN32
	#define MSG_NOSIGNAL 0
#endif

int Connection::send(Packet &packet) 
{
	int ret;

	if (fd == -1) return -1;

	ret = ::send(fd, (char*)&packet.len, sizeof(packet.len), MSG_NOSIGNAL);

	if (ret == -1) {
		send_error(__FUNCTION__, __LINE__);
		return -1;
	};

	ret = ::send(fd, (char*)packet.payload, packet.len, MSG_NOSIGNAL);
	
	if (ret == packet.len) return 0;
	if (ret == -1) {
		send_error(__FUNCTION__, __LINE__);
		perror("send");
		return -1;
	};

	if (ret < packet.len) {
		fprintf(stderr, "Not all data were send; this is a BUG!!!\n");
		return -1;
	};

	return -2;
};

Packet Connection::receive()
{
#ifdef WIN32
	unsigned long a=0;
	ioctlsocket(fd,FIONBIO,&a);
#endif

	int ret, size;
	Packet data;

	data.len = 0;
	data.payload = NULL;

	// If there is async receive in progress, don't allow this receive
	if (async_recv_len != -1) return data;
#ifdef WIN32
	ret = recv(fd, (char*)&data.len, sizeof(size), 0);
#else
	ret = recv(fd, (char*)&data.len, sizeof(size), MSG_WAITALL);
#endif
	

	if (ret == -1) 
		if (recv_error(__FUNCTION__, __LINE__)) {
			return data;
		} else {
			fprintf(stderr, "I'm too lazy to handle EAGAIN for length\n");
		};

		

	if (ret != sizeof(size)) {
		fprintf(stderr,"Something weird; I can't receive full length (ret is %d)\n", ret);
		return data;
	};
	
	data.payload = new char[data.len];
	size = data.len;
	while (size > 0) {
#ifdef WIN32
		ret = recv(fd, &((char*)data.payload)[data.len-size], size, 0);
#else
		ret = recv(fd, &((char*)data.payload)[data.len-size], size, MSG_WAITALL);
#endif

		if (ret == -1) {
			if (recv_error(__FUNCTION__, __LINE__)) {
				data.len = 0;
				delete static_cast<char*>(data.payload);
				data.payload = NULL;
				return data;
			};
			ret = 0;
		};

		size -= ret;
	};

	return data;
}

int Connection::receive_async(Packet &packet)
{
#ifdef WIN32
	unsigned long a=1;
	ioctlsocket(fd,FIONBIO,&a);
	#define MSG_DONTWAIT 0
	a=0;
#endif
	int ret, len;
	if (async_recv_len == -1) {
		int res=recv(fd, (char*)&len, sizeof(len), MSG_PEEK|MSG_DONTWAIT);

		if (res!= sizeof(len)){
#ifdef WIN32
			ioctlsocket(fd,FIONBIO,&a);
#endif
			return -1;
		}
	
		// we'll assume that receive will get all data needed
#ifdef WIN32
		ioctlsocket(fd,FIONBIO,&a);
		recv(fd, (char*)&async_recv_len, sizeof(len), 0);
#else
		recv(fd, (char*)&async_recv_len, sizeof(len), MSG_WAITALL);
#endif
	};

	packet.payload = new char[async_recv_len];
#ifdef WIN32
	a=1;
	ioctlsocket(fd,FIONBIO,&a);
#endif
	ret = recv(fd, (char*)packet.payload, async_recv_len, MSG_DONTWAIT|MSG_PEEK);

	if (ret == -1) {
		delete static_cast<char*>(packet.payload); packet.payload = NULL;
		// no data available at the moment
		if (recv_error(__FUNCTION__, __LINE__)) {
#ifdef WIN32
			shutdown(fd,SD_BOTH);
			closesocket(fd);
#else
			shutdown(fd, SHUT_RDWR);
			close(fd);
#endif
			fd = -1;
			
			return -1;
		}

		return -1;
	};

	if (ret != async_recv_len) {
		delete static_cast<char*>(packet.payload); packet.payload = NULL;
		return -1;
	};
#ifdef WIN32
	a=0;
	ioctlsocket(fd,FIONBIO,&a);
#endif
	ret = recv(fd, (char*)packet.payload, async_recv_len, MSG_DONTWAIT);

	if (ret == -1) {
		recv_error(__FUNCTION__, __LINE__);
		delete static_cast<char*>(packet.payload); packet.payload = NULL;
		//ioctlsocket(fd,FIONBIO,&a);
		return -1;
	};

	// packet successfully received
	packet.len = async_recv_len;
	async_recv_len = -1;
	//ioctlsocket(fd,FIONBIO,&a);
	return 0;
}

bool Connection::good()
{
#ifdef WIN32
	unsigned long ret;
	ioctlsocket(fd,FIONREAD,&ret);
	if(ret==WSAENETDOWN)return false;
	if(ret==WSAENOTSOCK)return false;
	return true;
#else
	struct pollfd p;

	if (fd == -1) return false;

	p.fd = fd;
	p.events = POLLERR|POLLHUP|POLLNVAL;

	poll(&p, 1, 0);
	
	if (p.revents > 0) {
		/* Error condition - clean up and make connection invalid */
		close(fd);
		fd = -1;
	};
	
	return fd > -1;
#endif
}

int Connection::recv_error(const char *fun, int line)
{
#ifdef WIN32
	#define EAGAIN WSAEWOULDBLOCK
	fprintf(stderr, "%s(%d): %d\n", fun, line, WSAGetLastError() );
	return 1;
#endif
	if (errno == EAGAIN) return 0;
	fprintf(stderr, "%s(%d):\n", fun, line );
	return 1;
};

int Connection::send_error(const char *fun, int line)
{
#ifdef WIN32
	#define EAGAIN WSAEWOULDBLOCK
	fprintf(stderr, "%s(%d): %d\n", fun, line, WSAGetLastError() );
	return 1;
#endif
	if (errno == EAGAIN) return 0;
	fprintf(stderr, "%s(%d):\n", fun, line );
	return 1;
};


/*
 * class NetworkServer
 */

NetworkServer::NetworkServer(){
};

void NetworkServer::init(string interface, int port)
{
#ifdef WIN32
	first_time_foo();

	struct sockaddr_in addr;

	// Create socket
	fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd == -1) perror("socket");

	// Bind it to specified port

	addr.sin_addr.s_addr=inet_addr(interface.c_str());
	
	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;

	if (bind( fd, (struct sockaddr *) &addr, sizeof(addr)) == -1 ) perror("bind");

	// Set it to listen state, backlog for 10 conns should be enough
	listen(fd, 10);
	unsigned long a;
	a=1;
	ioctlsocket(fd,FIONBIO,&a);

/*	struct addrinfo *result=NULL,hints;

	ZeroMemory(&hints,sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;
	hints.ai_flags = AI_PASSIVE;
	char buf[47];
	sprintf(buf,"%d",port);
	if(!getaddrinfo(NULL,buf,&hints,&result)){
		perror("get_addr_info");
		fd=-1;
		return;
	}

	fd=socket(result->ai_family,result->ai_protocol,result->ai_addrlen);
	if(fd==INVALID_SOCKET){
		perror("socket");
		fd=-1;
		return;
	}

	freeaddrinfo(result);

	if(listen(fd,10)==SOCKET_ERROR){
		perror("listen");
		return;
	}
*/
#else
	struct sockaddr_in addr;

	// Create socket
	fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd == -1) perror("socket");

	// Bind it to specified port

	inet_pton(AF_INET, interface.c_str(), &addr.sin_addr.s_addr);

	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;

	if (bind( fd, (struct sockaddr *) &addr, sizeof(addr)) == -1 ) perror("bind");

	// Set it to listen state, backlog for 10 conns should be enough
	listen(fd, 10);
#endif
}

NetworkServer::~NetworkServer()
{
#ifdef WIN32
	closesocket(fd);
#else
	close(fd);
#endif
}

bool NetworkServer::connection_pending()
{
#ifdef WIN32
	//IS IT OK?
	int socket = accept(fd,NULL,NULL);
	if(socket==INVALID_SOCKET)return false;
	this->accept_queue.push(socket);
	return true;
#else
	struct pollfd p;

	p.fd = fd;
	// Which one is on when connection is pending?
	p.events = POLLIN | POLLPRI;
	p.revents = 0;

	poll(&p, 1, 0);
	// TODO: implement
	return p.revents > 0;
#endif
}

Connection * NetworkServer::accept_connection()
{
#ifdef WIN32
	if(!this->accept_queue.empty()){
		int conn=this->accept_queue.front();
		accept_queue.pop();
		return new Connection(conn);
	}else{
		int conn = accept(fd,NULL,NULL);
		if(conn==INVALID_SOCKET){
			perror("HEY, YOU DONT CHECK IF SOMEONE KNOCKS ON THE DOOR!");
			return NULL;
		}
		return new Connection(conn);
	}
#else
	int conn = accept(fd, NULL, NULL);

	if (conn == -1) {
		perror("accept");
		return NULL;
	};

	return new Connection (conn);
#endif
}


