#include "headers.h"

extern int ROWS, COLS;
extern Mapa mapa;
extern vector<Player*> players;
extern vector<Missile*> missiles;
extern map<pair<int,int>, int> ammo;
extern Logger logger;

Player::Player()
{
	spree = 0;
	waiting = false;
	rot = 0;
	curweapon = 0;
	wastedtime = 0;
	weapons.push_back( new Pistol());
	weapons.push_back( new Brokovnica());
	weapons.push_back( new Bazooka());
	weapons.push_back( new Katusa());
	weapons.push_back( new Uzi());
	for(int i = 0; i < (int)weapons.size(); i++)
		weapons[i]->owner = this;
	hp=0; frags=deaths=0;
}

void Player::SendEnd()
{
	GameData data;
	data.type = TYPE_END;
	Packet packet;
	packet.payload = &data;
	packet.len = sizeof(data);
	conn->send(packet);
}

void Player::AddFrag()
{
	frags++;
	spree++;
	if(spree>10) spree=10;
	if(spree>=3)
	{
		SoundManager::PlaySound(spree-3+11);
	}
}

void Player::SendPlayerData()
{
	if(waiting) return;
	GameData data;
	data.type = TYPE_ROUND;
	data.x = x;
	data.y = y;
	data.rotate = rot;
	data.hp = hp;
	data.curweapon = curweapon;
	data.zasobnik = weapons[curweapon]->zasobnik;
	data.zbytok = weapons[curweapon]->zbytok;
	data.spawned = spawned;
	data.wastedtime = wastedtime;
	//data.canshot = weapons[curweapon]->CanShoot();
	data.blocktime = weapons[curweapon]->blocktime;
	mapa.GetVisiblePart(y, x, rot, data);
	data.nObjects = 0;
	logger.Output(MESSAGE, "Data r");
	for(int i = 0; i < (int)players.size(); i++)
	{
		if(players[i]==this) continue;
		if(data.mapa[players[i]->y][players[i]->x]==UNKNOWN) continue;
		data.objects[data.nObjects].type = OBJECT_PLAYER;
		data.objects[data.nObjects].x = players[i]->x;
		data.objects[data.nObjects].y = players[i]->y;
		data.objects[data.nObjects].rotate = players[i]->rot;
		data.objects[data.nObjects].id = i;
		data.nObjects++;
	}
	logger.Output(MESSAGE, "Data p");
	for(map<pair<int,int>, int>::iterator it = ammo.begin(); it!=ammo.end(); it++)
	{
		if(data.mapa[(*it).first.first][(*it).first.second]==UNKNOWN) continue;
		data.objects[data.nObjects].type = OBJECT_AMMO+(*it).second;
		data.objects[data.nObjects].x = (*it).first.second;
		data.objects[data.nObjects].y = (*it).first.first;
		data.objects[data.nObjects].id = -1;
		//data.objects[send.nObjects].spawned = true;
		data.nObjects++;
	}
	logger.Output(MESSAGE, "Data a");
	for(int i = 0; i < (int)missiles.size(); i++)
	{
		if(data.mapa[missiles[i]->y][missiles[i]->x]==UNKNOWN) continue;
		data.objects[data.nObjects].type = OBJECT_MISSILE+missiles[i]->type;
		data.objects[data.nObjects].x = missiles[i]->x;
		data.objects[data.nObjects].y = missiles[i]->y;
		data.objects[data.nObjects].rotate = missiles[i]->rot;
		data.objects[data.nObjects].id = missiles[i]->id+MAX_PLAYERS+3;
		data.nObjects++;
	}
	for(int i = 0; i < (int)weapons.size(); i++)
	{
		data.zasobniky[i]=weapons[i]->zasobnik;
		data.zbytky[i]=weapons[i]->zbytok;
	}
	logger.Output(MESSAGE, "Data m");
	Packet packet;
	packet.payload = &data;
	packet.len = sizeof(data);
	conn->send(packet);
	logger.Output(MESSAGE, "Data s");
	waiting = true;
}

bool Player::ReceivePlayerData(Action &act)
{
	Packet packet;
	act.mx=act.my=act.rotate=act.weaponaction=0;
	if(conn->receive_async(packet)==0)
	{
		wastedtime = 0;
		waiting = false;
		act = *((Action*)packet.payload);
		delete packet.payload;
		return true;
	}
	wastedtime++;
	return false;
}

void Player::SendInitData()
{
	Packet packet;
	GameData data;
	data.type = TYPE_INIT;
	data.y = ROWS;
	data.x = COLS;
	packet.payload = &data;
	packet.len = sizeof(data);
	conn->send(packet);
}

void Player::RecInitData()
{
	Packet p = conn->receive();
	ClientInit cinit = *((ClientInit*)p.payload);
	strcpy(nick, cinit.nick);
}

void Player::Spawn()
{
	hp = START_HP;
	vector<int> sp;
	bool g;
	for(int i = 0; i < (int)mapa.spawnx.size(); i++)
	{
		g = true;
		for(int j = 0; j < (int)players.size(); j++)
		{
			if(players[j]==this) continue;
			if(players[j]->y==mapa.spawny[i] && players[j]->x==mapa.spawnx[i]){
				g = false; break;}
		}
		if(g) sp.push_back(i);
	}
	int r = rand()%sp.size();
	x = mapa.spawnx[sp[r]];
	y = mapa.spawny[sp[r]];
	rot = rand()%16;
	spawned = true;
	for(int i = 0; i < (int)weapons.size(); i++)
		weapons[i]->Reset();
	curweapon = 0;
	movex=movey=0;
}

void Player::DeSpawn()
{
	spawned = false;
}