#ifndef WEAPON_H
#define WEAPON_H

class Player;

class Weapon
{
public:
	Player* owner;
	int zasobnik, zbytok;
	int zasobnik_size;
	int ammosize;
	int blocktime;
	int reloadtime;
	int shottime;
	Weapon() {};
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		if(blocktime > 0) return false;
		if(zasobnik <= 0) return false;
		blocktime = shottime;
		zasobnik--;
		return true;
	}
	virtual void Tick()
	{
		if(blocktime>0) blocktime--;
	}
	virtual bool Reload()
	{
		if(zasobnik >= zasobnik_size) return false;
		if(blocktime > 0) return false;
		int presun = min(zasobnik_size-zasobnik, zbytok);
		zasobnik+=presun;	
		zbytok-=presun;	
		blocktime = reloadtime;
		return true;
	}
	virtual void AddAmmo()
	{
		zbytok+=ammosize;
	}
	virtual bool CanShoot()
	{
		if(blocktime > 0) return false;
		if(zasobnik <= 0) return false;
		return true;
	}
	pair<int,int> GetAmmoCount(){return pair<int,int>(zasobnik, zbytok);}
	virtual void Reset()=0;
};

class Pistol : public Weapon
{
public:
	Pistol(){zasobnik_size=PISTOLZASOBNIK; zasobnik=PISTOLZASOBNIK; zbytok = 0; blocktime=0; shottime = PISTOLSHOTTIME; reloadtime=PISTOLRELOADTIME; ammosize=PISTOLAMMOSIZE;};
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		//printf("pistol\n");
		if(Weapon::Shot(missiles, y, x, rot)==false) return false;
		if(tras)
		{
			rot+=rand()%3-1;
			if(rot<0) rot+=16;
			rot%=16;
		}
		missiles.push_back(new PistolAmmo(y, x, rot, owner));
		SoundManager::PlaySound(0);
		return true;
	}
	virtual void Reset()
	{
		zasobnik=PISTOLZASOBNIK;
		zbytok=0;
		blocktime=0;
	}
};

class Uzi : public Weapon
{
public:
	int tr;
	Uzi(){zasobnik_size=UZIZASOBNIK; zasobnik=0; zbytok = 0; blocktime=0; shottime = UZISHOTTIME; reloadtime=UZIRELOADTIME; ammosize=UZIAMMOSIZE; tr=0;};
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		//printf("uzi\n");
		if(Weapon::Shot(missiles, y, x, rot)==false) return false;
		if(tras)
		{
			rot+=rand()%3-1;
			if(rot<0) rot+=16;
			rot%=16;
		}
		if(tr>0)
		{
			rot+=rand()%3-1;
			if(rot<0) rot+=16;
			rot%=16;
		}
		missiles.push_back(new PistolAmmo(y, x, rot, owner));
		tr=2;
		SoundManager::PlaySound(4);
		return true;
	}
	virtual void Tick()
	{
		Weapon::Tick();
		if(tr>0) tr--;
	}
	virtual void Reset()
	{
		zasobnik=0;
		zbytok=0;
		blocktime=0;
		tr=0;
	}
};

class Brokovnica : public Weapon
{
public:
	Brokovnica()
	{zasobnik_size=BROKZASOBNIK; zasobnik=BROKZASOBNIK; zbytok = 0; blocktime=0; shottime = BROKSHOTTIME; reloadtime=BROKRELOADTIME; ammosize=BROKAMMOSIZE;}
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		//printf("brok\n");
		if(Weapon::Shot(missiles, y, x, rot)==false) return false;
		missiles.push_back(new Brok(y, x, rot, owner));
		missiles.push_back(new Brok(y, x, (rot+2)%16, owner));
		missiles.push_back(new Brok(y, x, (rot+1)%16, owner));
		missiles.push_back(new Brok(y, x, (rot+15)%16, owner));
		missiles.push_back(new Brok(y, x, (rot+14)%16, owner));
		SoundManager::PlaySound(1);
		return true;
	}
	virtual void Reset()
	{
		zasobnik=BROKZASOBNIK;
		zbytok=0;
		blocktime=0;
	}
};

class Bazooka : public Weapon
{
public:
	Bazooka(){zasobnik_size=BAZOOKAZASOBNIK; zasobnik=BAZOOKAZASOBNIK; zbytok = 0; blocktime=0; shottime = BAZOOKASHOTTIME; reloadtime=BAZOOKARELOADTIME; ammosize=BAZOOKAAMMOSIZE;};
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		//printf("bazooka\n");
		if(Weapon::Shot(missiles, y, x, rot)==false) return false;
		if(tras)
		{
			rot+=rand()%3-1;
			if(rot<0) rot+=16;
			rot%=16;
		}
		missiles.push_back(new Rocket(y, x, rot, owner));
		SoundManager::PlaySound(2);
		return true;
	}
	virtual void Reset()
	{
		zasobnik=BAZOOKAZASOBNIK;
		zbytok=0;
		blocktime=0;
	}
};

class Katusa : public Weapon
{
public:
	Katusa(){zasobnik_size=KATUSAZASOBNIK; zasobnik=0; zbytok = 0; blocktime=0; shottime = KATUSASHOTTIME; reloadtime=KATUSARELOADTIME; ammosize=KATUSAAMMOSIZE;};
	virtual bool Shot(vector<Missile*>& missiles, int y, int x, int rot, bool tras=false)
	{
		//printf("bazooka\n");
		if(Weapon::Shot(missiles, y, x, rot)==false) return false;
		if(tras)
		{
			rot+=rand()%3-1;
			if(rot<0) rot+=16;
			rot%=16;
		}
		missiles.push_back(new KRocket(y, x, rot, owner));
		SoundManager::PlaySound(3);
		return true;
	}
	virtual void Reset()
	{
		zasobnik=0;
		zbytok=0;
		blocktime=0;
	}
};

#endif